from enum import Enum


class CardNetwork(str, Enum):
    atm = 'atm'
    visa = 'visa'


class CardTransactionType(str, Enum):
    auth = 'auth'
    capture = 'capture'
    expiration = 'expiration'
    refund = 'refund'
    void = 'void'


class CardType(str, Enum):
    physical = 'physical'
    virtual = 'virtual'


class DepositNetwork(str, Enum):
    cash = 'cash'
    internal = 'internal'
    spei = 'spei'


class Status(str, Enum):
    created = 'created'
    submitted = 'submitted'
    in_review = 'in_review'
    succeeded = 'succeeded'
    failed = 'failed'


class TransferNetwork(str, Enum):
    internal = 'internal'
    spei = 'spei'



__all__ = [
    '__version__',
    'ApiKeyQuery',
    'CardNetwork',
    'CardTransactionType',
    'CardType',
    'ClientRequestParams',
    'DepositNetwork',
    'DictStrAny',
    'Limit',
    'OptionalDict',
    'PaymentCardNumber',
    'QueryParams',
    'SantizedDict',
    'Status',
    'StrictPaymentCardNumber',
    'StrictPositiveInt',
    'StrictTransferRequest',
    'TransactionQuery',
    'TransferNetwork',
    'TransferQuery',
    'TransferRequest',
    'sanitize_dict',
]

from .types import (
    ApiKeyQuery,
    CardNetwork,
    CardTransactionType,
    CardType,
    DepositNetwork,
    Limit,
    PaymentCardNumber,
    QueryParams,
    SantizedDict,
    Status,
    StrictPaymentCardNumber,
    StrictPositiveInt,
    StrictTransferRequest,
    TransactionQuery,
    TransferNetwork,
    TransferQuery,
    TransferRequest,
)
from .typing import ClientRequestParams, DictStrAny, OptionalDict
from .validators import sanitize_dict
from .version import __version__



import datetime as dt

import pytest

from cuenca_validations import QueryParams, SantizedDict, Status


def test_sanitized_dict():
    now = dt.datetime.now()
    assert SantizedDict(
        status=Status.succeeded, time=now, hello='there'
    ) == dict(status='succeeded', time=now.isoformat(), hello='there')


@pytest.mark.parametrize(
    'count, truth',
    [
        (1, True),
        ('1', True),
        (True, True),
        (False, False),
        (0, False),
        ('0', False),
    ],
)
def test_count(count, truth):
    q = QueryParams(count=count)
    assert q.count is truth


def test_dict():
    now = dt.datetime.utcnow()
    model = QueryParams(count=1, created_before=now)
    assert model.dict() == dict(count=1, created_before=now.isoformat())



import datetime as dt
from typing import Optional

from pydantic import BaseModel, Extra
from pydantic.types import ConstrainedInt

from ..typing import DictStrAny
from ..validators import sanitize_dict

MAX_PAGE_LIMIT = 100


class Limit(ConstrainedInt):
    gt = 0
    le = MAX_PAGE_LIMIT


class QueryParams(BaseModel):
    count: bool = False
    limit: Optional[Limit] = None
    user: Optional[str] = None
    created_before: Optional[dt.datetime] = None
    created_after: Optional[dt.datetime] = None

    class Config:
        extra = Extra.forbid  # raise ValidationError if there are extra fields

    def dict(self, *args, **kwargs) -> DictStrAny:
        d = super().dict(exclude_none=True, exclude_unset=True)
        if self.count:
            d['count'] = 1
        sanitize_dict(d)
        return d


class TransactionQuery(QueryParams):
    status: Optional[str] = None


class TransferQuery(TransactionQuery):
    account_number: Optional[str] = None
    idempotency_key: Optional[str] = None


class ApiKeyQuery(QueryParams):
    active: Optional[bool] = None



from pydantic import PositiveInt, StrictInt

from ..validators import sanitize_dict


class SantizedDict(dict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        sanitize_dict(self)


class StrictPositiveInt(StrictInt, PositiveInt):
    """
    - StrictInt: ensures a float isn't passed in by accident
    - PositiveInt: ensures the value is above 0
    """

    ...



from typing import Any, Dict, MutableMapping, Optional, Union

ClientRequestParams = Union[None, bytes, MutableMapping[str, str]]
DictStrAny = Dict[str, Any]
OptionalDict = Optional[Dict[str, Union[int, str]]]



from typing import Union

from clabe import Clabe
from pydantic import BaseModel, StrictStr

from .card import PaymentCardNumber, StrictPaymentCardNumber
from .general import StrictPositiveInt


class TransferRequest(BaseModel):
    recipient_name: StrictStr
    account_number: Union[Clabe, PaymentCardNumber]
    amount: StrictPositiveInt  # in centavos
    descriptor: StrictStr  # how it'll appear for the recipient
    idempotency_key: str  # must be unique for each transfer


class StrictTransferRequest(TransferRequest):
    account_number: Union[Clabe, StrictPaymentCardNumber]



__all__ = [
    'ApiKeyQuery',
    'CardNetwork',
    'CardTransactionType',
    'CardType',
    'DepositNetwork',
    'Limit',
    'PaymentCardNumber',
    'QueryParams',
    'SantizedDict',
    'Status',
    'StrictPaymentCardNumber',
    'StrictPositiveInt',
    'StrictTransferRequest',
    'TransactionQuery',
    'TransferNetwork',
    'TransferQuery',
    'TransferRequest',
]

from .card import PaymentCardNumber, StrictPaymentCardNumber
from .enums import (
    CardNetwork,
    CardTransactionType,
    CardType,
    DepositNetwork,
    Status,
    TransferNetwork,
)
from .general import SantizedDict, StrictPositiveInt
from .queries import (
    ApiKeyQuery,
    Limit,
    QueryParams,
    TransactionQuery,
    TransferQuery,
)
from .requests import StrictTransferRequest, TransferRequest



__all__ = ['sanitize_dict']

import datetime as dt
from enum import Enum


def sanitize_dict(d: dict):
    for k, v in d.items():
        if isinstance(v, dt.date):
            d[k] = v.isoformat()
        elif isinstance(v, Enum):
            d[k] = v.value



from pydantic.errors import PydanticValueError


class CardBinValidationError(PydanticValueError):
    code = 'payment_card_number.bin'
    msg_template = (
        'The card number contains a BIN (first six digits) that does not have'
        'a known association with a Mexican bank. To add the association,'
        'please file an issue:'
        'https://github.com/cuenca-mx/cuenca-validations/issues'
    )
